import logging
from io import TextIOWrapper
from typing import List

from shortlinks.utils import (
    check_url,
    export_vanity_map_json,
    flatten_links,
    read_file,
    write_redirect_html,
)


class ShortlinkGenerator:
    logger = logging.getLogger("shortlinks.ShortlinkGenerator")
    link_mappings: dict[str, str] = {}
    vanity_file: TextIOWrapper
    ignore_urls: List[str]

    def __init__(self, vanity_file: TextIOWrapper, ignore_urls: List[str]):
        self.vanity_file = vanity_file
        self.link_mappings = read_file(vanity_file)
        self.ignore_urls = ignore_urls

    def check_all_urls(self) -> bool:
        flat_map = flatten_links(self.link_mappings)
        broken_links = 0
        self.logger.info("Checking URLs...")
        for path, url in flat_map.items():
            if url is None:
                continue
            if not check_url(url, self.ignore_urls):
                self.logger.error(f"^ Error for path {path}")
                broken_links = broken_links + 1
        self.logger.warning(f"Found {broken_links} broken links.")
        if broken_links > 0:
            return False
        return True

    def generate(self, out_path: str) -> None:
        flat_map = flatten_links(self.link_mappings)
        export_vanity_map_json(out_path, self.link_mappings)
        for path, url in flat_map.items():
            if url:
                self.logger.info(f"Writing redirect for {path}")
                check_url(url, self.ignore_urls)
                write_redirect_html(out_path, path, url)
